package ddejonge.nb3;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;

/**
 * Base class for the NB3Logger class.
 * Tool for debugging.
 * 
 * @author Dave de Jonge
 *
 */
class Logger {
	
 	String folderPath;		//e.g. ddjonge/Documents/files/results/
	String fileName;     	//e.g. myNewExperiment.log
	String filePath;
 	public boolean enabled = false;
 	private ArrayList<String> errorInfo;
 	
 	
 	int capacity = 1000;
 	
 	ArrayList<String> content = new ArrayList<String>(capacity);
 	
	Logger(){		 		

	}
	
	Logger(int _capacity){
		capacity = _capacity;
	}
	
	
	void logln(){
		logln("");
	}
	 
	 void logln(String s){
		 logln(s, false);
	 }
	 
	//Adds a new line to the log.
	void logln(String s, boolean print){
		
		if(!enabled){return;}
		
		if(print){
			System.out.println(s);
		}
		
		content.add(s);
	}
	
	
	

	
	
	/**appends text to the last line in the log
	 * 
	 * @param s
	 * @param print
	 */
	void log(String s){
		log(s, false);
	}
	
	/**appends text to the last line in the log
	 * 
	 * @param s
	 * @param print
	 */
	void log(String s, boolean print){
	
		if(!enabled){return;}
		
		if(print){
			System.out.print(s);
		}
		
		String s2 = content.get(content.size()-1);
		s2 += s;
		content.remove(content.size()-1);
		content.add(s2);
		
	}
	

	
	void log(int n){	
		
		log(Integer.toString(n));
	}
	
	
	void log(int n, boolean print){
		
		log(Integer.toString(n), print);
	}
	
	
	void println(String s){
		if(!enabled){return;}
		
		System.out.println(s);
		
	}
	
	void println(int n){
		if(!enabled){return;}
		
		System.out.println(Integer.toString(n));
		
	}
	
	
	void clearErrorInfo(){
		errorInfo = new ArrayList<String>();
	}
	
	void addErrorInfo(String s){
		errorInfo.add(s);
	}
	
	void displayErrorInfo(){
		System.out.println();
		System.out.println("*Error Info: *");
		for(String s : errorInfo){
			System.out.println(s);
		}
		System.out.println();
	}
	
	
	
	void writeToFile(){
				
		if(!enabled){return;}
		
		//create the folder
		File folder = new File(folderPath);
		if (!folder.exists()) {
			folder.mkdirs();
		}
		
		//create the file
		File file = new File(folder, fileName);
		if(!file.exists()){
			try {
				file.createNewFile();
			} catch (IOException e) {
				
				System.out.println("Logger.writeToFile() folderPath: " + folderPath);
				System.out.println("Logger.writeToFile() fileName: " + fileName);
				
				e.printStackTrace();
				return;
			}
		}
		
		PrintWriter out = null;
		try {
			out = new PrintWriter(new FileWriter(file, true));
	
			for(String s : content){
				out.println(s);
			}	
			
			content.clear();
			
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			if(out != null){
				out.flush();
				out.close();
			}
		}
		

		
	}
	
	
	
	
	 static String getDateString(){
			//Get the current time to put in the filename of the log file
			Calendar now = Calendar.getInstance();
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd__HH-mm-ss");
			return sdf.format(now.getTime());
	 }
}
