package ddejonge.nb3;


import java.util.ArrayList;
import java.util.List;


/**
 * Class used to represent messages internally in the NB3 algorithm.
 * 
 * Should be converted to and from a class that represents a message in the domain of application.
 * 
 * @author Dave de Jonge
 *
 */
public class NB3Message {

	public static enum MsgType {PROPOSE, ACCEPT, ILLEGAL, CONFIRM, REJECT};
	
	//FIELDS
	String sender;
	List<String> receivers;	
	
	/**
	 * Propose, accept, reject...
	 */
	MsgType type;   			
	
	/**
	 * The content of the message. That is: the actions that the sender proposes to the receivers.
	 */
	List<NB3Action> actions;
	
	
	/**
	 * An identifier to identify the deal that is being proposed / accepted. This ensures that we don't have to call an equals() method on the list of actions.
	 */
	String proposalID;
	
	
	
	//CONSTRUCTOR	
	/**
	 * Constructor to create a message object that can be understood by the NB3 algorithm.
	 * 
	 * Whenever the agent receives a message according to some negotiation/communication protocol, that message should be converted
	 * to an object of the class NB3Message, so that it can be handled by the NB3 algorithm.
	 * 
	 * @param sender
	 * @param receivers
	 * @param type
	 * @param actions
	 * @param proposalID
	 */
	public NB3Message(String sender, List<String> receivers, MsgType type, List<? extends NB3Action> actions, String proposalID){
		
		this.sender = sender;
		this.receivers = receivers;
		this.type = type;
		this.actions = new ArrayList<>(actions);
		this.proposalID = proposalID;
		
	}
	
	
	
	
	//GETTERS
	public MsgType getType(){ 
		return type;
	}
	
	
	public List<? extends NB3Action> getActions(){
		return new ArrayList<>(actions);
	}
	
	
	public String getSender(){
		return sender;
	} 
	
	
	public String getProposalID(){
		return proposalID;
	}
	


}
