package ddejonge.nb3;
import java.util.ArrayList;








public class NB3Proposal implements Comparable<NB3Proposal>{


	static int proposalsMade = 0;
	
	final static int MY_NORM_UTILITY_INIT_VAL = -1000;
	
	//FIELDS
	int creationTime;
	int discoverTime = -1;
	private int proposerID;   		/**id of the agent who made this proposal.*/
	private String id;			    /**id of the proposal*/
	int correspondingNodeId;  		/**id of the node in the tree that corresponds to this proposal*/
	
	
	/** 
	 * This array stores for each agent the normalized utility it would receive from this deal. 
	 *  It is simply copied from the node passed to the constructor. 
	 */ 
	float[] normalizedUtility;
	
	/** The normalized utility for the agent that runs this algorithm. */
	float myNormalizedUtility = MY_NORM_UTILITY_INIT_VAL; //We initialize it with some value to check later whether it has been set or not. 
	
	/** The absolute value each agent would receive from this deal. */
	float[] e;
	
			//notice that Node doesn't have a field called myNormalizedUtility. Proposal does have it, because it is
			//difficult to retrieve myNumber from a Proposal object.
	float oppNormalizedUtility = -1;				//copied from Node it is used to 
	
	/** The actions that this proposal consists of. */
	private ArrayList<NB3Action> actions = new ArrayList<NB3Action>(10);   	
		
	/** The agents participating in this proposal. */
	private NB3AgentSet pa = new NB3AgentSet();	 	 						
	
	/** The set of agents that have not yet replied to this proposal.*/
	NB3AgentSet waitingFor = new NB3AgentSet(); 						
	
	boolean isValid = true;  //check for internal consistency. currently we do not perform this check, so this value is always true.
	
	/** Is set to true when the agent on which this algorithm is running accepts the deal */
	boolean isAcceptedByMe = false;
	
	int confirmationTime = -1;
	

	//CONSTRUCTORS
	/**
	 * To construct a proposal from a node (either a node directly found by the search algorithm, or a node added to represent an incoming proposal)
	 * 
	 * @param n
	 * @param discTime
	 * @param proposer
	 * @param sessionInfo
	 */
	NB3Proposal(NB3Node n, int discTime, int proposer, NB3SessionInfo sessionInfo){
		
		creationTime = sessionInfo.getCurrentTime();
		discoverTime = discTime;
		
		this.normalizedUtility = new float[sessionInfo.numAgents];	
		this.e = new float[sessionInfo.numAgents];

		
		proposalsMade++;
		id = sessionInfo.agentNames[proposer] + proposalsMade;
		correspondingNodeId = n.nodeId;
		
		//copy the bounds and the satisfactions
		for(int i=0; i<sessionInfo.numAgents; i++){
			normalizedUtility[i] = n.getNormalizedUtility(i);
			e[i] = n.e[i];
		}
		myNormalizedUtility  = n.getMyNormalizedUtility();
		oppNormalizedUtility = n.getOpponentUtility();
		
		//Warning: we have to be sure that the normalized utilities have already been calculated!		
		if(myNormalizedUtility == MY_NORM_UTILITY_INIT_VAL){
			throw new IllegalArgumentException("Nb3Proposal.Nb3Proposal() Error! normalized utilities have not been calculated.");
		}
		
		
		actions.addAll(n.theTree.theAlgorithm.branch2actions(n.getBranch()));
		pa.addAll(n.theTree.theAlgorithm.getParticipatingAgents(n.getBranch()));
		
		
		//***Set the list of agents that we are still waiting for.
		waitingFor.addAll(pa);
		waitingFor.remove(proposer);
		

		
		//***Finally make sure the deal is valid.
		//validate();
	}
	
	
	
	/**
	 * This constructor is called when we receive a CONFIRM message for a deal in which we are not participating.
	 * (since we are not participating it hasn't been created during our search)
	 * 
	 * @param confirmMessage
	 * @param sessionInfo
	 */
	NB3Proposal(NB3Message confirmMessage, NB3SessionInfo sessionInfo) {
			
		this.creationTime = sessionInfo.getCurrentTime();
		//discoverTime = Globals.msgContent2discoverTime(msg.getContent()); 
		
		this.normalizedUtility = new float[sessionInfo.numAgents];	
		this.e = new float[sessionInfo.numAgents];
		
		this.id = confirmMessage.getProposalID();
		
		//***Set the list of actions
		this.actions = (ArrayList<NB3Action>) confirmMessage.getActions();
		
		//***Set the participating agents
		for(NB3Action action : actions){
			if(action.getParticipatingAgents(sessionInfo) != null){
				this.pa.addAll(action.getParticipatingAgents(sessionInfo));	
			}
		
		}
		
	}

	
	
	
	
	
	//ACCESS METHODS
	public ArrayList<? extends NB3Action> getActions(){
		return actions;
	}
	
	public NB3AgentSet getParticipatingAgents(){		
		return pa;		
	}
	
	
	void setID(String id){
		this.id = id;
	}
	
	public String getID(){
		return id;
	}
	
	public int getProposer() {
		return proposerID;
	}

	public String toMessageString(){		
		
		String string = actions.get(0).toString();		
		
		for(int i=1; i<actions.size(); i++){			
			string += "#" + actions.get(i).toString();			
		}
		
		string += "#dt" + discoverTime;
		
		return string;
	}
	
	public void setAcceptedBy(int acceptingAgent) {
		waitingFor.remove(acceptingAgent);	
	}
	
	public boolean isAccepted(){
		return (waitingFor.isEmpty());
	}
	
	public double getMyNormalizedUtility(){
		return myNormalizedUtility;
	}
	
	public double getOppNormalizedUtility(){
		
		return oppNormalizedUtility;			
		
	}
	
	public String toString(){
		
		String string = actions.get(0).toString();		
		
		for(int i=1; i<actions.size(); i++){			
			string += "\r\n" + actions.get(i).toString();			
		}
		
		return string;
	}
	
	
	public int compareTo(NB3Proposal other){
		
		if(this.myNormalizedUtility > other.myNormalizedUtility){
			return -1;
		}else if(this.myNormalizedUtility < other.myNormalizedUtility){
			return 1;
		}else{
			return 0;
		}
		
	}
}
