package ddejonge.nb3;


import java.util.HashMap;


/**
 * Class to holds information about the negotiation domain, such as the names of the agents, the time
 * that negotiations started, and the deadline of the negotiations.
 * 
 * 
 * @author Dave de Jonge
 *
 */
class NB3SessionInfo {

	//FIELDS
	
	/**number of agents involved in the MAS*/
	final int numAgents;
	
	/**Maps the ID of each agent to the name of that agent.*/
	String[] agentNames;
		
	/**Maps the name of each agent to the ID of that agent.*/
	private HashMap<String, Integer> name2int = new HashMap<>();
	
	
	/**
	 * The time at which the negotiations started.
	 * This is a Unix-timestamp, so it represents the absolute date and time at which negotiations end (e.g. 10 January 2017 14:00).
	 */
	long startTime;  	
	
	
	/**
	 * The time at which the negotiations stop.
	 * This is a Unix-timestamp, so it represents the absolute date and time at which negotiations end (e.g. 10 January 2017 14:00).
	 */
	long deadLine;
	
	
	
	
	
	
	//CONSTRUCTOR
	NB3SessionInfo(String myName, String[] agentNames, long deadline){
		
		this.numAgents = agentNames.length;
		this.agentNames = agentNames.clone();
		
		for(int i=0; i<numAgents; i++){
			name2int.put(agentNames[i], i);
		}
		
		this.deadLine = deadline;
	}
	
	
	/**
	 * Returns the amount of milliseconds that have passed since the beginning of negotiations.
	 */
	int getCurrentTime(){
		return (int) (System.currentTimeMillis() - startTime);
	}

	
	
	public int getAgentID(String name){
		
		if(name == null){
			throw new RuntimeException("NB3SessionInfo.getAgentID() Error! name == null");
		}
		
		Integer agentID = name2int.get(name);
		
		if(agentID == null){
			throw new RuntimeException("NB3SessionInfo.getAgentID() Error! name not found: " + name);
		}
		
		return agentID;
	}
}
